__author__ = 'Udayakumar Rajan'

from enum import Enum, auto
from typing import Dict, Type

from ..interface.EncryptionBase import EncryptionBase
from ..services.asymmetric_encryption import AsymmetricEncryption
from ..services.config_service import ConfigService
from ..services.symmetric_encryption import SymmetricEncryption
from ..utils.logger import get_logger

logger = get_logger(__name__)

class EncryptionType(Enum):
    SYMMETRIC = "symmetric"
    ASYMMETRIC = "asymmetric"

class EncryptionFactory:
    _encryption_mapping: Dict[EncryptionType, Type[EncryptionBase]] = {
        EncryptionType.SYMMETRIC: SymmetricEncryption,
        EncryptionType.ASYMMETRIC: AsymmetricEncryption,
    }

    @classmethod
    def get_encryption_service(cls, encryption_type: str) -> EncryptionBase:
        """
        Create and return an encryption service instance based on the encryption type.
        
        Args:
            encryption_type: The type of encryption service to create.
        
        Returns:
            An instance of the requested encryption service.
            
        Raises:
            ValueError: If an invalid encryption type is provided.
        """
        try:
            enum_type = EncryptionType(encryption_type)
            logger.info(f"Creating {enum_type.value} EncryptionService with registered Secret Manager URL.")
            return cls._encryption_mapping[enum_type]()
        except (ValueError, KeyError):
            logger.error(f"Invalid encryption type: {encryption_type}")
            raise ValueError("Invalid encryption type specified. Expected values are 'symmetric' or 'asymmetric'.")

