__author__ = 'Udayakumar Rajan'

from abc import ABC, abstractmethod
from typing import Dict, Optional, Any


class EncryptionBase(ABC):
    """
    Base class for encryption and decryption operations.
    
    This abstract class defines the interface for encryption and decryption
    operations, supporting both symmetric (shared secret) and asymmetric (RSA)
    encryption methods.
    """

    @abstractmethod
    async def encrypt(
        self,
        payload: Dict[str, Any],
        api_key: str,
        secret_key: str,
        shared_secret: Optional[str] = None,
        private_key: Optional[str] = None
    ) -> str:
        """
        Abstract method for encrypting payload data.

        Args:
            payload: Dictionary containing the data to encrypt
            api_key: API key for authentication
            secret_key: Secret key for encryption
            shared_secret: Optional shared secret for symmetric encryption
            private_key: Optional private key for asymmetric encryption

        Returns:
            str: Encrypted payload as a JWE string
        """
        raise NotImplementedError

    @abstractmethod
    async def decrypt(
        self,
        jwe: str,
        api_key: str,
        shared_secret: Optional[str] = None,
        public_key: Optional[str] = None
    ) -> str:
        """
        Abstract method for decrypting JWE data.

        Args:
            jwe: JWE string to decrypt
            api_key: API key for authentication
            shared_secret: Optional shared secret for symmetric decryption
            public_key: Optional public key for asymmetric decryption

        Returns:
            str: Decrypted payload
        """
        raise NotImplementedError

    async def encrypt_with_shared_secret(
        self,
        payload: Dict[str, Any],
        api_key: str,
        shared_secret: Optional[str] = None
    ) -> str:
        """
        Encrypt payload using symmetric encryption.

        Args:
            payload: Dictionary containing the data to encrypt
            api_key: API key for authentication
            shared_secret: Optional shared secret for encryption

        Returns:
            str: Encrypted payload as a JWE string
        """
        return await self.encrypt(payload, api_key, secret_key="", shared_secret=shared_secret)

    async def encrypt_with_rsa(
        self,
        payload: Dict[str, Any],
        api_key: str,
        private_key: str
    ) -> str:
        """
        Encrypt payload using asymmetric encryption.

        Args:
            payload: Dictionary containing the data to encrypt
            api_key: API key for authentication
            private_key: Private key for RSA encryption

        Returns:
            str: Encrypted payload as a JWE string
        """
        return await self.encrypt(payload, api_key, secret_key="", private_key=private_key)

    async def decrypt_with_shared_secret(
        self,
        encrypted_payload: str,
        api_key: str,
        shared_secret: Optional[str] = None
    ) -> str:
        """
        Decrypt payload using symmetric encryption.

        Args:
            encrypted_payload: JWE string to decrypt
            api_key: API key for authentication
            shared_secret: Optional shared secret for decryption

        Returns:
            str: Decrypted payload
        """
        return await self.decrypt(encrypted_payload, api_key, shared_secret=shared_secret)

    async def decrypt_with_rsa(
        self,
        encrypted_payload: str,
        api_key: str,
        public_key: Optional[str] = None
    ) -> str:
        """
        Decrypt payload using asymmetric encryption.

        Args:
            encrypted_payload: JWE string to decrypt
            api_key: API key for authentication
            public_key: Public key for RSA decryption

        Returns:
            str: Decrypted payload
        """
        return await self.decrypt(encrypted_payload, api_key, public_key=public_key)
