__author__ = 'Udayakumar Rajan'

from threading import Lock
from typing import Optional
import re
from urllib.parse import urlparse

from ..utils.logger import get_logger

logger = get_logger(__name__)


class ConfigServiceError(Exception):
    """Base exception for ConfigService operations."""
    pass


class ConfigService:
    """
    Singleton service for managing configuration settings.
    Thread-safe implementation for handling secret manager URL configuration.
    """
    
    _instance = None
    _lock = Lock()
    _secret_manager_url: Optional[str] = None
    _url_pattern = re.compile(
        r'^https?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    def __new__(cls) -> 'ConfigService':
        with cls._lock:
            if not cls._instance:
                cls._instance = super(ConfigService, cls).__new__(cls)
            return cls._instance

    @classmethod
    def _validate_url(cls, url: str) -> None:
        """
        Validate URL format.
        
        Args:
            url: URL string to validate
            
        Raises:
            ValueError: If URL is invalid
        """
        if not url:
            raise ValueError("Secret Manager URL cannot be empty")
        
        if not cls._url_pattern.match(url):
            raise ValueError("Invalid URL format")
        
        try:
            result = urlparse(url)
            if not all([result.scheme, result.netloc]):
                raise ValueError("Invalid URL format")
        except Exception as e:
            raise ValueError(f"Invalid URL: {str(e)}")

    def register_secret_manager_url(self, url: str) -> None:
        """
        Register the secret manager URL.
        
        Args:
            url: Secret manager URL to register
            
        Raises:
            ValueError: If URL is invalid
        """
        self._validate_url(url)
        
        with self._lock:
            self._secret_manager_url = url
            logger.info("Registered Secret Manager URL: %s", url)

    def get_secret_manager_url(self) -> str:
        """
        Get the registered secret manager URL.
        
        Returns:
            str: The registered secret manager URL
            
        Raises:
            ConfigServiceError: If URL is not registered
        """
        with self._lock:
            if not self._secret_manager_url:
                logger.error("Attempted to access unregistered Secret Manager URL")
                raise ConfigServiceError("Secret Manager URL is not registered")
            return self._secret_manager_url

