__author__ = 'Udayakumar Rajan'

import json
import time
from typing import Any, Dict, Union

from jwcrypto import jwe, jwk
from jwcrypto.common import JWException

from ..utils.constant import JWE_HEADER_ALG, JWE_HEADER_ENC, ChannelSecurityContext
from ..utils.logger import get_logger

logger = get_logger(__name__)


class CryptoServiceError(Exception):
    """Base exception for crypto service operations."""
    pass


class CryptoService:
    """Service for handling JWE encryption and decryption operations."""

    @staticmethod
    def generate_jwe_with_sharedsecret(
        payload: Union[Dict[str, Any], str],
        secret_key: str,
        shared_secret: str,
        channel_security_context: str
    ) -> str:
        """
        Generate a JWE using specified algorithms for encryption and key wrapping.

        Args:
            payload: Data to encrypt (dictionary or string)
            secret_key: Key identifier for the encryption
            shared_secret: Secret key for encryption
            channel_security_context: Security context for the channel

        Returns:
            str: Serialized JWE token

        Raises:
            CryptoServiceError: If encryption fails
        """
        try:
            # Validate inputs
            if not all([payload, secret_key, shared_secret, channel_security_context]):
                raise ValueError("All parameters are required")

            # Convert dict payload to string if necessary
            payload_str = json.dumps(payload) if isinstance(payload, dict) else str(payload)

            # Create a JWK object from the symmetric key
            key = jwk.JWK(kty='oct', k=shared_secret)

            # Create the JWE header
            header = {
                "alg": JWE_HEADER_ALG,
                "enc": JWE_HEADER_ENC,
                "cty": "JWE",
                "typ": "JOSE",
                "channelSecurityContext": channel_security_context,
                "kid": secret_key,
                "iat": str(int(time.time()))
            }

            # Create and encrypt JWE
            jwe_obj = jwe.JWE(payload_str.encode(), protected=header)
            jwe_obj.add_recipient(key)
            
            return jwe_obj.serialize(compact=True)

        except (ValueError, TypeError) as e:
            logger.error("Invalid input parameters: %s", str(e))
            raise CryptoServiceError(f"Invalid input parameters: {str(e)}") from e
        except JWException as e:
            logger.error("JWE encryption failed: %s", str(e))
            raise CryptoServiceError(f"JWE encryption failed: {str(e)}") from e
        except Exception as e:
            logger.error("Unexpected error during encryption: %s", str(e))
            raise CryptoServiceError(f"Encryption failed: {str(e)}") from e

    @staticmethod
    def decrypt_jwe_sharedsecret(jwe_token: str, shared_secret: str) -> str:
        """
        Decrypt a JWE token using the provided symmetric key.

        Args:
            jwe_token: JWE token to decrypt
            shared_secret: Secret key for decryption

        Returns:
            str: Decrypted payload

        Raises:
            CryptoServiceError: If decryption fails
        """
        try:
            # Validate inputs
            if not jwe_token or not shared_secret:
                raise ValueError("JWE token and shared secret are required")

            # Create a JWK object from the symmetric key
            key = jwk.JWK(kty='oct', k=shared_secret)

            # Create and decrypt JWE
            jwe_obj = jwe.JWE()
            jwe_obj.deserialize(jwe_token)
            jwe_obj.decrypt(key)

            return jwe_obj.payload.decode('utf-8')

        except (ValueError, TypeError) as e:
            logger.error("Invalid input parameters: %s", str(e))
            raise CryptoServiceError(f"Invalid input parameters: {str(e)}") from e
        except JWException as e:
            logger.error("JWE decryption failed: %s", str(e))
            raise CryptoServiceError(f"JWE decryption failed: {str(e)}") from e
        except Exception as e:
            logger.error("Unexpected error during decryption: %s", str(e))
            raise CryptoServiceError(f"Decryption failed: {str(e)}") from e
