__author__ = 'Udayakumar Rajan'

from enum import Enum
from typing import Dict, Any, Optional

from ..factory.encryption_factory import EncryptionFactory
from ..services.secret_service import SecretService
from ..utils.logger import get_logger

logger = get_logger(__name__)


class EncryptionType(Enum):
    """Supported encryption types."""
    SYMMETRIC = "Symmetric"
    ASYMMETRIC = "Asymmetric"


class EncryptionServiceError(Exception):
    """Base exception for encryption service operations."""
    pass


class EncryptionService:
    """Facade for encryption and decryption operations."""

    @staticmethod
    async def encrypt(
        payload: Dict[str, Any],
        api_key: str,
        encryption_type: str = "",
        shared_secret: Optional[str] = None,
        private_key: Optional[str] = None
    ) -> str:
        """
        Encrypt payload using specified encryption type.

        Args:
            payload: Data to encrypt
            api_key: API key for authentication
            encryption_type: Type of encryption (Symmetric/Asymmetric)
            shared_secret: Optional shared secret for symmetric encryption
            private_key: Optional private key for asymmetric encryption

        Returns:
            str: Encrypted payload

        Raises:
            EncryptionServiceError: If encryption fails
            ValueError: If invalid encryption type provided
        """
        try:
            if not payload or not api_key:
                raise ValueError("Payload and API key are required")

            # Validate encryption type
            try:
                enc_type = EncryptionType(encryption_type)
            except ValueError:
                valid_types = ", ".join([t.value for t in EncryptionType])
                raise ValueError(f"Invalid encryption_type: {encryption_type}. Must be one of: {valid_types}")

            logger.info("Initiating encryption with type: %s", enc_type.value)
            encryption_service = EncryptionFactory.get_encryption_service(enc_type.value.lower())
            
            return await encryption_service.encrypt(
                payload=payload,
                api_key=api_key,
                secret_key="",  # This will be fetched by the service if needed
                shared_secret=shared_secret,
                private_key=private_key
            )

        except ValueError as e:
            logger.error("Validation error: %s", str(e))
            raise
        except Exception as e:
            logger.error("Encryption failed: %s", str(e))
            raise EncryptionServiceError(f"Encryption failed: {str(e)}") from e

    @staticmethod
    async def decrypt(encrypted_payload: str, api_key: str) -> str:
        """
        Decrypt encrypted payload.

        Args:
            encrypted_payload: Encrypted data to decrypt
            api_key: API key for authentication

        Returns:
            str: Decrypted payload

        Raises:
            EncryptionServiceError: If decryption fails
        """
        try:
            if not encrypted_payload or not api_key:
                raise ValueError("Encrypted payload and API key are required")

            logger.info("Fetching encryption type from secrets")
            secrets = await SecretService.fetch_shared_secret(api_key)
            
            if 'type' not in secrets:
                raise ValueError("Encryption type not found in secrets")
                
            encryption_type = secrets['type']
            logger.info("Initiating decryption with type: %s", encryption_type)
            
            encryption_service = EncryptionFactory.get_encryption_service(encryption_type.lower())
            return await encryption_service.decrypt(encrypted_payload, api_key)

        except ValueError as e:
            logger.error("Validation error: %s", str(e))
            raise
        except Exception as e:
            logger.error("Decryption failed: %s", str(e))
            raise EncryptionServiceError(f"Decryption failed: {str(e)}") from e
