__author__ = 'Udayakumar Rajan'
import os
import httpx
from websockets.legacy.framing import encode_data

from ..utils.encryption_util import EncryptionUtil
from ..utils.logger import get_logger
from ..services.config_service import ConfigService

logger = get_logger(__name__)

SHARED_SECRET_URL = os.getenv("SHARED_SECRET_URL", "/encryption-service/api/v1/secrets/get-shared-secret")

class SecretService :
    @staticmethod
    async def fetch_shared_secret (api_key:str, secret_key:str) -> str:
        """
                Fetch shared secret from a secret manager service using the provided API key.

                Args:
                    api_key (str): The API key to validate the application.
                    secret_key (str) : The secret to retrieve the secret

                Returns:
                    str: The shared secret value.
                    :param secret_key:
                """
        try:
            # Initialize ConfigService and get the secret manager URL
            config_service = ConfigService()
            secret_manager_url = config_service.get_secret_manager_url()

            # Construct the full URL with query parameter
            url = f"{secret_manager_url}{SHARED_SECRET_URL}"
            logger.info(f"Calling secret manager with URL: {url}")
            headers = {"x-api-key": api_key, "x-secret-key": secret_key}
            # Make the asynchronous HTTP request to fetch the secret
            async with httpx.AsyncClient() as client:
                response = await client.get(url, headers=headers)

            # Check if the request was successful
            if response.status_code == 200:
                secret_data = response.json()
                encode_data = secret_data.get("sharedSecret", "")
                key = f"{api_key}{secret_key}"
                shared_secret = await EncryptionUtil.decrypt_and_base64_decode(encode_data, key)
                logger.info(f"Successfully fetched shared secret for API key: {api_key}")
                return shared_secret
            else:
                logger.error(f"Failed to fetch shared secret. Status code: {response.status_code}")
                raise Exception(f"Failed to fetch shared secret. Status code: {response.status_code}")
        except Exception as e:
            logger.error(f"Error fetching shared secret: {str(e)}")
            raise Exception(f"Error fetching shared secret: {str(e)}")
