__author__ = 'Udayakumar Rajan'

from typing import Dict, Optional

from ..interface.EncryptionBase import EncryptionBase
from ..utils.encryption_util import EncryptionUtil
from ..utils.logger import get_logger
from .crypto_service import CryptoService
from .secret_service import SecretService


logger = get_logger(__name__)


class SymmetricEncryptionError(Exception):
    """Base exception for symmetric encryption operations."""
    pass


class SharedSecretError(SymmetricEncryptionError):
    """Exception raised for shared secret related errors."""
    pass


class SymmetricEncryption(EncryptionBase):
    """Implementation of symmetric encryption and decryption."""

    @staticmethod
    async def _get_shared_secret(api_key: str, secret_key: str, shared_secret: Optional[str]) -> str:
        """
        Get shared secret from parameter or fetch from service.
        
        Args:
            api_key: API key for authentication
            secret_key: Secret key for fetching shared secret
            shared_secret: Optional pre-existing shared secret
            
        Returns:
            str: The shared secret to use for encryption/decryption
            
        Raises:
            SharedSecretError: If shared secret cannot be fetched
        """
        if shared_secret:
            return shared_secret
        try:
            return await SecretService.fetch_shared_secret(api_key, secret_key)
        except Exception as e:
            logger.error(f"Failed to fetch shared secret: {e}")
            raise SharedSecretError(f"Unable to get shared secret: {str(e)}") from e

    async def encrypt(
        self,
        payload: Dict,
        api_key: str,
        secret_key: str,
        shared_secret: Optional[str] = None,
        private_key: Optional[str] = None  # Kept for interface compatibility
    ) -> str:
        """
        Encrypt the payload using symmetric encryption.
        """
        if not payload or not api_key or not secret_key:
            raise ValueError("Missing required parameters: payload, api_key, or secret_key")

        try:
            secret = await self._get_shared_secret(api_key, secret_key, shared_secret)
            logger.info("Encrypting payload using symmetric encryption.")
            
            return CryptoService.generate_jwe_with_sharedsecret(
                payload=payload,
                secret_key=secret_key,
                shared_secret=secret,
                channel_security_context="SHARED_SECRET"
            )
        except SharedSecretError:
            raise
        except Exception as e:
            logger.error(f"Error during symmetric encryption: {e}")
            raise SymmetricEncryptionError("Encryption failed") from e

    async def decrypt(
        self,
        jwe: str,
        api_key: str,
        shared_secret: Optional[str] = None,
        public_key: Optional[str] = None  # Kept for interface compatibility
    ) -> str:
        """
        Decrypt the encrypted payload using symmetric decryption.
        """
        if not jwe or not api_key:
            raise ValueError("Missing required parameters: jwe or api_key")

        try:
            header = EncryptionUtil.decode_jwe_header(jwe)
            kid = header.get("kid")
            if not kid:
                raise ValueError("KID value is missing in the header")

            secret = await self._get_shared_secret(api_key, kid, shared_secret)
            logger.info("Decrypting payload using symmetric decryption.")
            
            return CryptoService.decrypt_jwe_sharedsecret(jwe, secret)
        except SharedSecretError:
            raise
        except Exception as e:
            logger.error(f"Error during symmetric decryption: {e}")
            raise SymmetricEncryptionError("Decryption failed") from e
