import base64
import json
import os
from http.client import HTTPException
from hashlib import sha256
from Crypto.Cipher import AES
from Crypto.Util.Padding import unpad

from jwcrypto import  jwk
DEFAULT_KEY_SIZE = 32  # 256 bits

class EncryptionUtil:
    @staticmethod
    def base64url_encode(data):
        """
        Base64url encode the input bytes.
        """
        return base64.urlsafe_b64encode(data).rstrip(b'=').decode('utf-8')

    @staticmethod
    def base64url_decode(data):
        """
        Decode a base64url-encoded string to bytes.
        """
        padding = '=' * (4 - len(data) % 4)  # Add the necessary padding for base64 decoding
        return base64.urlsafe_b64decode(data + padding)

    @staticmethod
    def generate_api_key () :
        """ Utility to generate a random API Key """
        return EncryptionUtil.base64url_encode(os.urandom(37))

    # Utility to generate a random shared secret
    @staticmethod
    def generate_shared_secret():
        """ Utility to generate a random shared secret """
        key = jwk.JWK(kty='oct', k=EncryptionUtil.base64url_encode(os.urandom(DEFAULT_KEY_SIZE)))
        return key.k  # 256-bit (32-byte) shared secret

    @staticmethod
    async def get_channel_security_context(jwe_token: str) -> HTTPException:
        """
        Extracts the `channelSecurityContext` value from the header of a JWE token.

        Args:
            jwe_token (str): The JWE token.

        Returns:
            str: The `channelSecurityContext` value, or None if it doesn't exist.
        """
        try:
            # Split the JWE token to get the header segment
            header_segment = jwe_token.split('.')[0]

            # Decode the Base64Url-encoded header
            header_bytes = base64.urlsafe_b64decode(header_segment + "===")  # Add padding if needed
            header_json = header_bytes.decode('utf-8')

            # Parse the header JSON
            header = json.loads(header_json)

            # Return the channelSecurityContext value
            return header.get("channelSecurityContext")
        except Exception as e:
            # Handle any errors (e.g., malformed token, decoding issues)
            print(f"Error extracting channelSecurityContext: {e}")
            return HTTPException("Required param is missing in the header", str(e))

    @staticmethod
    async def decrypt_and_base64_decode(encoded_data: str, secret_key: str) -> str:
        """
        Decrypt the Base64 encoded data using AES and the provided secret key.

        :param encoded_data: The Base64 encoded encrypted data.
        :param secret_key: The secret key used for decryption (must match the one used during encryption).
        :return: The decrypted data.
        """
        key = await EncryptionUtil.generate_32byte_key(secret_key)
        await EncryptionUtil.validate_key_length(key)  # Validate key length

        # Convert Base64url to Base64
        encoded_data = EncryptionUtil.base64url_to_base64(encoded_data)
        try:
            # Decode the Base64 encoded data
            encrypted_data = base64.b64decode(encoded_data)
        except Exception as e:
            raise ValueError("Error during Base64 decoding.") from e

        # Extract the IV (first 16 bytes) and encrypted data (the rest)
        iv = encrypted_data[:16]
        encrypted_data = encrypted_data[16:]

        cipher = AES.new(key, AES.MODE_CBC, iv)

        # Decrypt the data and unpad it
        decrypted_data = unpad(cipher.decrypt(encrypted_data), AES.block_size)

        return decrypted_data.decode()

    @staticmethod
    async def generate_32byte_key(input_string: str) -> bytes:
        return sha256(input_string.encode()).digest()

    # Function to check if the key length is valid
    @staticmethod
    async def validate_key_length(key: bytes):
        if len(key) not in [16, 24, 32]:
            raise ValueError("Invalid key length. Key must be 16, 24, or 32 bytes.")

    @staticmethod
    def base64url_to_base64(encoded_data: str) -> str:
        """
        Converts Base64url to standard Base64 by replacing `-` with `+` and `_` with `/`.
        """
        encoded_data = encoded_data.replace('-', '+').replace('_', '/')
        # Add padding if necessary
        missing_padding = len(encoded_data) % 4
        if missing_padding:
            encoded_data += '=' * (4 - missing_padding)
        return encoded_data

    @staticmethod
    def decode_jwe_header(jwe_string):
        # Split the JWE string into its components
        parts = jwe_string.split('.')
        if len(parts) < 5:
            raise ValueError("Invalid JWE string")

        # The first part is the Base64URL-encoded header
        encoded_header = parts[0]

        # Decode Base64URL to get the JSON header
        # Replace Base64URL-specific characters to make it Base64 standard-compliant
        padded_header = encoded_header + '=' * (4 - len(encoded_header) % 4)
        decoded_header = base64.urlsafe_b64decode(padded_header)

        # Parse the JSON header
        header = json.loads(decoded_header)
        return header
